@@ -82,19 +82,13 @@ using namespace studio;
 
 /* === G L O B A L S ======================================================= */
 
-extern const guint gtk_major_version;
-extern const guint gtk_minor_version;
-extern const guint gtk_micro_version;
-extern const guint gtk_binary_age;
-extern const guint gtk_interface_age;
-
 /* === P R O C E D U R E S ================================================= */
 
 /* === M E T H O D S ======================================================= */
 
 About::About()
 {
-
+	set_transient_for((Gtk::Window&)(*App::main_window));
 #ifdef HAVE_GTK_ABOUTDIALOG_SET_PROGRAM_NAME
 	set_program_name(PACKAGE_NAME);
 #else
@@ -103,11 +97,12 @@ About::About()
 	set_version(VERSION);
 	set_comments(_("2D vector animation studio"));
 
-	set_url_hook(sigc::mem_fun(*this, &About::on_link_clicked));
+	// TODO: set_url_hook bot works in GTK3
+	//set_url_hook(sigc::mem_fun(*this, &About::on_link_clicked));
 	set_website("http://synfig.org/");
 	set_website_label(_("Visit the Synfig website"));
 
-	set_copyright(_("Copyright 2001-2014\nRobert B. Quattlebaum Jr.,\nAdrian Bentley and Synfig contributors"));
+	set_copyright(_("Copyright 2001-2013\nRobert B. Quattlebaum Jr.,\nAdrian Bentley and Synfig contributors"));
 	Glib::ustring license =
 		"This program is free software; you can redistribute it and/or modify "
 		"it under the terms of the GNU General Public License as published by "
@@ -146,8 +141,10 @@ About::About()
 	authors.push_back("Daniel Fort");
 	authors.push_back("Daniel Hornung (rubikcube)");
 	authors.push_back("David Roden (Bombe)");
+	authors.push_back("Denis Zdorovtsov (trizer)");
 	authors.push_back("Dmitriy Pomerantsev (Atrus)");
 	authors.push_back("Douglas Lau");
+	authors.push_back("Evgenij Katunov");
 	authors.push_back("Gerald Young (Yoyobuae)");
 	authors.push_back("Gerco Ballintijn");
 	authors.push_back("IL'dar AKHmetgaleev (AkhIL)");
@@ -160,6 +157,7 @@ About::About()
 	authors.push_back("Miguel Gea Milvaques (xerakko)");
 	authors.push_back("Paul Wise (pabs)");
 	authors.push_back("Ralf Corsepius");
+	authors.push_back("Ramon Miranda");
 	authors.push_back("Ray Frederikson");
 	authors.push_back("Timo Paulssen (timonator)");
 	authors.push_back("Yu Chen (jcome)");
