@@ -30,12 +30,9 @@
 #	include <config.h>
 #endif
 
-#include <gtkmm/dialog.h>
-#include <gtkmm/entry.h>
-
-#include <synfig/valuenode_dynamiclist.h>
+#include <synfig/valuenodes/valuenode_dynamiclist.h>
 #include <synfigapp/action_system.h>
-#include <synfig/valuenode_bline.h>
+#include <synfig/valuenodes/valuenode_bline.h>
 
 #include "state_circle.h"
 #include "state_normal.h"
@@ -46,11 +43,11 @@
 #include <synfigapp/action.h>
 #include "event_mouse.h"
 #include "event_layerclick.h"
-#include "toolbox.h"
+#include "docks/dock_toolbox.h"
 #include "docks/dialog_tooloptions.h"
-#include <gtkmm/optionmenu.h>
 #include "duck.h"
 #include "widgets/widget_enum.h"
+#include "widgets/widget_distance.h"
 #include <synfigapp/main.h>
 
 #include "general.h"
@@ -75,6 +72,29 @@ enum CircleFalloff
 	CIRCLE_NUM_FALLOFF
 };
 
+#ifndef LAYER_CREATION
+#define LAYER_CREATION(button, stockid, tooltip)	\
+	{ \
+		Gtk::Image *icon = manage(new Gtk::Image(Gtk::StockID(stockid), \
+			Gtk::ICON_SIZE_SMALL_TOOLBAR)); \
+		button.add(*icon); \
+	} \
+	button.set_relief(Gtk::RELIEF_NONE); \
+	button.set_tooltip_text(tooltip) ;\
+	button.signal_toggled().connect(sigc::mem_fun(*this, \
+		&studio::StateCircle_Context::toggle_layer_creation))
+#endif
+
+// indentation for options layout
+#ifndef SPACING
+#define SPACING(name, px) \
+	Gtk::Alignment *name = Gtk::manage(new Gtk::Alignment()); \
+	name->set_size_request(px)
+#endif
+
+#define GAP	(3)
+#define INDENTATION (6)
+
 /* === G L O B A L S ======================================================= */
 
 StateCircle studio::state_circle;
@@ -99,32 +119,74 @@ class studio::StateCircle_Context : public sigc::trackable
 	//Toolbox settings
 	synfigapp::Settings& settings;
 
-	//Toolbox display
+	// holder of options
 	Gtk::Table options_table;
 
-	Gtk::Entry		entry_id; //what to name the layer
+	// title
+	Gtk::Label title_label;
 
-	Widget_Enum		enum_falloff;
-#ifdef BLEND_METHOD_IN_TOOL_OPTIONS
-	Widget_Enum		enum_blend;
-#endif	// BLEND_METHOD_IN_TOOL_OPTIONS
+	// layer name:
+	Gtk::Label id_label;
+	Gtk::HBox id_box;
+	Gtk::Entry id_entry;
 
-	Gtk::Adjustment	adj_feather;
-	Gtk::Adjustment	adj_number_of_bline_points;
-	Gtk::Adjustment	adj_bline_point_angle_offset;
-	Gtk::SpinButton	spin_feather;
-	Gtk::SpinButton	spin_number_of_bline_points;
-	Gtk::SpinButton	spin_bline_point_angle_offset;
+	// layer types to create:
+	Gtk::Label layer_types_label;
+	Gtk::ToggleButton layer_circle_togglebutton;
+	Gtk::ToggleButton layer_region_togglebutton;
+	Gtk::ToggleButton layer_outline_togglebutton;
+	Gtk::ToggleButton layer_advanced_outline_togglebutton;
+	Gtk::ToggleButton layer_curve_gradient_togglebutton;
+	Gtk::ToggleButton layer_plant_togglebutton;
+	Gtk::HBox layer_types_box;
 
-	Gtk::CheckButton checkbutton_invert;
-	Gtk::CheckButton checkbutton_layer_circle;
-	Gtk::CheckButton checkbutton_layer_region;
-	Gtk::CheckButton checkbutton_layer_outline;
-	Gtk::CheckButton checkbutton_layer_advanced_outline;
-	Gtk::CheckButton checkbutton_layer_curve_gradient;
-	Gtk::CheckButton checkbutton_layer_plant;
-	Gtk::CheckButton checkbutton_layer_link_origins;
-	Gtk::CheckButton checkbutton_layer_origins_at_center;
+	// blend method
+	Gtk::Label blend_label;
+	Gtk::HBox blend_box;
+	Widget_Enum blend_enum;
+
+	// opacity
+	Gtk::Label opacity_label;
+	Gtk::HScale opacity_hscl;
+
+	// brush size
+	Gtk::Label bline_width_label;
+	Widget_Distance bline_width_dist;
+
+	// spline points
+	Gtk::Label bline_points_label;
+	Glib::RefPtr<Gtk::Adjustment> number_of_bline_points_adj;
+	Gtk::SpinButton	number_of_bline_points_spin;
+
+	// spline point angle offset
+	Gtk::Label bline_point_angle_offset_label;
+	Glib::RefPtr<Gtk::Adjustment> bline_point_angle_offset_adj;
+	Gtk::SpinButton	bline_point_angle_offset_spin;
+	Gtk::HBox bline_point_angle_offset_box;
+
+	// invert
+	Gtk::Label invert_label;
+	Gtk::CheckButton invert_checkbutton;
+	Gtk::HBox invert_box;
+
+	// feather size
+	Gtk::Label feather_label;
+	Widget_Distance feather_dist;
+
+	// falloff of feather of circle layer
+	Gtk::Label falloff_label;
+	Gtk::HBox falloff_box;
+	Widget_Enum falloff_enum;
+
+	// link origins
+	Gtk::Label link_origins_label;
+	Gtk::CheckButton layer_link_origins_checkbutton;
+	Gtk::HBox link_origins_box;
+
+	// spline origins at center
+	Gtk::Label origins_at_center_label;
+	Gtk::CheckButton layer_origins_at_center_checkbutton;
+	Gtk::HBox origins_at_center_box;
 
 public:
 
@@ -140,52 +202,74 @@ public:
 			get_layer_plant_flag();
 	}
 
-	synfig::String get_id()const { return entry_id.get_text(); }
-	void set_id(const synfig::String& x) { return entry_id.set_text(x); }
+	synfig::String get_id()const { return id_entry.get_text(); }
+	void set_id(const synfig::String& x) { return id_entry.set_text(x); }
 
-	int get_falloff()const { return enum_falloff.get_value(); }
-	void set_falloff(int x) { return enum_falloff.set_value(x); }
+	int get_falloff()const { return falloff_enum.get_value(); }
+	void set_falloff(int x) { return falloff_enum.set_value(x); }
 
-#ifdef BLEND_METHOD_IN_TOOL_OPTIONS
-	int get_blend()const { return enum_blend.get_value(); }
-	void set_blend(int x) { return enum_blend.set_value(x); }
-#endif	// BLEND_METHOD_IN_TOOL_OPTIONS
+	int get_blend()const { return blend_enum.get_value(); }
+	void set_blend(int x) { return blend_enum.set_value(x); }
 
-	Real get_feather()const { return adj_feather.get_value(); }
-	void set_feather(Real f) { adj_feather.set_value(f); }
+	Real get_opacity()const { return opacity_hscl.get_value(); }
+	void set_opacity(Real x) { opacity_hscl.set_value(x); }
 
-	Real get_number_of_bline_points()const { return adj_number_of_bline_points.get_value(); }
-	void set_number_of_bline_points(Real f) { adj_number_of_bline_points.set_value(f); }
+	Real get_bline_width() const {
+		return bline_width_dist.get_value().get(
+			Distance::SYSTEM_UNITS,
+			get_canvas_view()->get_canvas()->rend_desc()
+		);
+	}
+	void set_bline_width(Distance x) { return bline_width_dist.set_value(x);}
 
-	Real get_bline_point_angle_offset()const { return adj_bline_point_angle_offset.get_value(); }
-	void set_bline_point_angle_offset(Real f) { adj_bline_point_angle_offset.set_value(f); }
+	Real get_feather_size() const {
+		return feather_dist.get_value().get(
+			Distance::SYSTEM_UNITS,
+			get_canvas_view()->get_canvas()->rend_desc()
+		);
+	}
+	void set_feather_size(Distance x) { return feather_dist.set_value(x);}
 
-	bool get_invert()const { return checkbutton_invert.get_active(); }
-	void set_invert(bool i) { checkbutton_invert.set_active(i); }
+	Real get_number_of_bline_points()const { return number_of_bline_points_adj->get_value(); }
+	void set_number_of_bline_points(Real f) { number_of_bline_points_adj->set_value(f); }
 
-	bool get_layer_circle_flag()const { return checkbutton_layer_circle.get_active(); }
-	void set_layer_circle_flag(bool x) { return checkbutton_layer_circle.set_active(x); }
+	Real get_bline_point_angle_offset()const { return bline_point_angle_offset_adj->get_value(); }
+	void set_bline_point_angle_offset(Real f) { bline_point_angle_offset_adj->set_value(f); }
 
-	bool get_layer_region_flag()const { return checkbutton_layer_region.get_active(); }
-	void set_layer_region_flag(bool x) { return checkbutton_layer_region.set_active(x); }
+	bool get_invert()const { return invert_checkbutton.get_active(); }
+	void set_invert(bool i) { invert_checkbutton.set_active(i); }
 
-	bool get_layer_outline_flag()const { return checkbutton_layer_outline.get_active(); }
-	void set_layer_outline_flag(bool x) { return checkbutton_layer_outline.set_active(x); }
+  bool pre_layer_circle_flag();
+	bool get_layer_circle_flag()const { return layer_circle_togglebutton.get_active(); }
+	void set_layer_circle_flag(bool x) { return layer_circle_togglebutton.set_active(x); }
 
-	bool get_layer_advanced_outline_flag()const { return checkbutton_layer_advanced_outline.get_active(); }
-	void set_layer_advanced_outline_flag(bool x) { return checkbutton_layer_advanced_outline.set_active(x); }
+	bool get_layer_region_flag()const { return layer_region_togglebutton.get_active(); }
+	void set_layer_region_flag(bool x) { return layer_region_togglebutton.set_active(x); }
 
-	bool get_layer_curve_gradient_flag()const { return checkbutton_layer_curve_gradient.get_active(); }
-	void set_layer_curve_gradient_flag(bool x) { return checkbutton_layer_curve_gradient.set_active(x); }
+	bool get_layer_outline_flag()const { return layer_outline_togglebutton.get_active(); }
+	void set_layer_outline_flag(bool x) { return layer_outline_togglebutton.set_active(x); }
 
-	bool get_layer_plant_flag()const { return checkbutton_layer_plant.get_active(); }
-	void set_layer_plant_flag(bool x) { return checkbutton_layer_plant.set_active(x); }
+	bool get_layer_advanced_outline_flag()const { return layer_advanced_outline_togglebutton.get_active(); }
+	void set_layer_advanced_outline_flag(bool x) { return layer_advanced_outline_togglebutton.set_active(x); }
 
-	bool get_layer_link_origins_flag()const { return checkbutton_layer_link_origins.get_active(); }
-	void set_layer_link_origins_flag(bool x) { return checkbutton_layer_link_origins.set_active(x); }
+	bool get_layer_curve_gradient_flag()const { return layer_curve_gradient_togglebutton.get_active(); }
+	void set_layer_curve_gradient_flag(bool x) { return layer_curve_gradient_togglebutton.set_active(x); }
 
-	bool get_layer_origins_at_center_flag()const { return checkbutton_layer_origins_at_center.get_active(); }
-	void set_layer_origins_at_center_flag(bool x) { return checkbutton_layer_origins_at_center.set_active(x); }
+	bool get_layer_plant_flag()const { return layer_plant_togglebutton.get_active(); }
+	void set_layer_plant_flag(bool x) { return layer_plant_togglebutton.set_active(x); }
+
+	bool get_layer_link_origins_flag()const { return layer_link_origins_checkbutton.get_active(); }
+	void set_layer_link_origins_flag(bool x) { return layer_link_origins_checkbutton.set_active(x); }
+
+	bool get_layer_origins_at_center_flag()const { return layer_origins_at_center_checkbutton.get_active(); }
+	void set_layer_origins_at_center_flag(bool x) { return layer_origins_at_center_checkbutton.set_active(x); }
+
+  bool layer_circle_flag;
+  bool layer_region_flag;
+  bool layer_outline_flag;
+  bool layer_advanced_outline_flag;
+  bool layer_curve_gradient_flag;
+  bool layer_plant_flag;
 
 	void refresh_tool_options(); //to refresh the toolbox
 
@@ -220,6 +304,8 @@ public:
 
 	void make_circle(const Point& p1, const Point& p2);
 
+	void toggle_layer_creation();
+
 };	// END of class StateCircle_Context
 
 /* === M E T H O D S ======================================================= */
@@ -260,17 +346,25 @@ StateCircle_Context::load_settings()
 		else
 			set_falloff(2);
 
-	#ifdef BLEND_METHOD_IN_TOOL_OPTIONS
 		if(settings.get_value("circle.blend",value) && value != "")
 			set_blend(atoi(value.c_str()));
 		else
 			set_blend(0);//(int)Color::BLEND_COMPOSITE); //0 should be blend composites value
-	#endif	// BLEND_METHOD_IN_TOOL_OPTIONS
+
+		if(settings.get_value("circle.opacity",value))
+			set_opacity(atof(value.c_str()));
+		else
+			set_opacity(1);
+
+		if(settings.get_value("circle.bline_width",value) && value != "")
+			set_bline_width(Distance(atof(value.c_str()), App::distance_system));
+		else
+			set_bline_width(Distance(1, App::distance_system)); // default width
 
 		if(settings.get_value("circle.feather",value))
-			set_feather(atof(value.c_str()));
+			set_feather_size(Distance(atof(value.c_str()), App::distance_system));
 		else
-			set_feather(0);
+			set_feather_size(Distance(0, App::distance_system)); // default feather
 
 		if(settings.get_value("circle.number_of_bline_points",value))
 			set_number_of_bline_points(atof(value.c_str()));
@@ -326,6 +420,15 @@ StateCircle_Context::load_settings()
 			set_layer_origins_at_center_flag(false);
 		else
 			set_layer_origins_at_center_flag(true);
+
+  // determine layer flags
+	layer_circle_flag = get_layer_circle_flag();
+  layer_region_flag = get_layer_region_flag();
+  layer_outline_flag = get_layer_outline_flag();
+  layer_advanced_outline_flag = get_layer_outline_flag();
+  layer_curve_gradient_flag = get_layer_curve_gradient_flag();
+  layer_plant_flag = get_layer_plant_flag();
+
 	}
 	catch(...)
 	{
@@ -341,10 +444,10 @@ StateCircle_Context::save_settings()
 		synfig::ChangeLocale change_locale(LC_NUMERIC, "C");
 		settings.set_value("circle.id",get_id());
 		settings.set_value("circle.fallofftype",strprintf("%d",get_falloff()));
-	#ifdef BLEND_METHOD_IN_TOOL_OPTIONS
 		settings.set_value("circle.blend",strprintf("%d",get_blend()));
-	#endif	// BLEND_METHOD_IN_TOOL_OPTIONS
-		settings.set_value("circle.feather",strprintf("%f",(float)get_feather()));
+		settings.set_value("circle.opacity",strprintf("%f",(float)get_opacity()));
+		settings.set_value("circle.bline_width", bline_width_dist.get_value().get_string());
+		settings.set_value("circle.feather", feather_dist.get_value().get_string());
 		settings.set_value("circle.number_of_bline_points",strprintf("%d",(int)(get_number_of_bline_points() + 0.5)));
 		settings.set_value("circle.bline_point_angle_offset",strprintf("%f",(float)get_bline_point_angle_offset()));
 		settings.set_value("circle.invert",get_invert()?"1":"0");
@@ -420,27 +523,136 @@ StateCircle_Context::StateCircle_Context(CanvasView* canvas_view):
 	duckmatic_push(get_work_area()),
 	prev_workarea_layer_status_(get_work_area()->get_allow_layer_clicks()),
 	settings(synfigapp::Main::get_selected_input_device()->settings()),
-	entry_id(),				//   value lower upper  step page
-	adj_feather(					0,    0,    1, 0.01, 0.1),
-	adj_number_of_bline_points(		0,    2,  120, 1   , 1  ),
-	adj_bline_point_angle_offset(	0, -360,  360, 0.1 , 1  ),
-	spin_feather(adj_feather,0.1,3),
-	spin_number_of_bline_points(adj_number_of_bline_points,1,0),
-	spin_bline_point_angle_offset(adj_bline_point_angle_offset,1,1),
-	checkbutton_invert(_("Invert")),
-	checkbutton_layer_circle(_("Create Circle Layer")),
-	checkbutton_layer_region(_("Create Region")),
-	checkbutton_layer_outline(_("Create Outline")),
-	checkbutton_layer_advanced_outline(_("Create Advanced Outline")),
-	checkbutton_layer_curve_gradient(_("Create Curve Gradient")),
-	checkbutton_layer_plant(_("Create Plant")),
-	checkbutton_layer_link_origins(_("Link Origins")),
-	checkbutton_layer_origins_at_center(_("Spline Origins at Center"))
+	opacity_hscl(0.0f, 1.01f, 0.01f),
+	number_of_bline_points_adj(Gtk::Adjustment::create(0, 2, 120, 1, 1)),
+	number_of_bline_points_spin(number_of_bline_points_adj, 1, 0),
+	bline_point_angle_offset_adj(Gtk::Adjustment::create(0, -360, 360, 0.1, 1)),
+	bline_point_angle_offset_spin(bline_point_angle_offset_adj, 1, 1)
 {
 	egress_on_selection_change=true;
 
-	// Set up the tool options dialog
-	enum_falloff.set_param_desc(ParamDesc("falloff")
+
+	/* Set up the tool options dialog */
+
+	// 0, title
+	title_label.set_label(_("Circle Creation"));
+	Pango::AttrList list;
+	Pango::AttrInt attr = Pango::Attribute::create_attr_weight(Pango::WEIGHT_BOLD);
+	list.insert(attr);
+	title_label.set_attributes(list);
+	title_label.set_alignment(Gtk::ALIGN_START, Gtk::ALIGN_CENTER);
+
+	// 1, layer name label and entry
+	id_label.set_label(_("Name:"));
+	id_label.set_alignment(Gtk::ALIGN_START, Gtk::ALIGN_CENTER);
+	SPACING(id_gap, GAP);
+	id_box.pack_start(id_label, Gtk::PACK_SHRINK);
+	id_box.pack_start(*id_gap, Gtk::PACK_SHRINK);
+
+	id_box.pack_start(id_entry);
+
+	// 2, layer types creation
+	layer_types_label.set_label(_("Create:"));
+	layer_types_label.set_alignment(Gtk::ALIGN_START, Gtk::ALIGN_CENTER);
+
+	LAYER_CREATION(layer_circle_togglebutton,
+		("synfig-layer_geometry_circle"), _("Create a circle layer"));
+
+	LAYER_CREATION(layer_region_togglebutton,
+		("synfig-layer_geometry_region"), _("Create a region layer"));
+
+	LAYER_CREATION(layer_outline_togglebutton,
+		("synfig-layer_geometry_outline"), _("Create a outline layer"));
+
+	LAYER_CREATION(layer_advanced_outline_togglebutton,
+		("synfig-layer_geometry_advanced_outline"), _("Create a advanced outline layer"));
+
+	LAYER_CREATION(layer_plant_togglebutton,
+		("synfig-layer_other_plant"), _("Create a plant layer"));
+
+	LAYER_CREATION(layer_curve_gradient_togglebutton,
+		("synfig-layer_gradient_curve"), _("Create a gradient layer"));
+
+	SPACING(layer_types_indent, INDENTATION);
+
+	layer_types_box.pack_start(*layer_types_indent, Gtk::PACK_SHRINK);
+	layer_types_box.pack_start(layer_circle_togglebutton, Gtk::PACK_SHRINK);
+	layer_types_box.pack_start(layer_region_togglebutton, Gtk::PACK_SHRINK);
+	layer_types_box.pack_start(layer_outline_togglebutton, Gtk::PACK_SHRINK);
+	layer_types_box.pack_start(layer_advanced_outline_togglebutton, Gtk::PACK_SHRINK);
+	layer_types_box.pack_start(layer_plant_togglebutton, Gtk::PACK_SHRINK);
+	layer_types_box.pack_start(layer_curve_gradient_togglebutton, Gtk::PACK_SHRINK);
+
+	// 3, blend method label and dropdown list
+	blend_label.set_label(_("Blend Method:"));
+	blend_label.set_alignment(Gtk::ALIGN_START, Gtk::ALIGN_CENTER);
+	SPACING(blend_gap, GAP);
+	blend_box.pack_start(blend_label, Gtk::PACK_SHRINK);
+	blend_box.pack_start(*blend_gap, Gtk::PACK_SHRINK);
+
+	blend_enum.set_param_desc(ParamDesc(Color::BLEND_COMPOSITE,"blend_method")
+		.set_local_name(_("Blend Method"))
+		.set_description(_("Defines the blend method to be used for circles")));
+
+	// 4, opacity label and slider
+	opacity_label.set_label(_("Opacity:"));
+	opacity_label.set_alignment(Gtk::ALIGN_START, Gtk::ALIGN_CENTER);
+
+	opacity_hscl.set_digits(2);
+	opacity_hscl.set_value_pos(Gtk::POS_LEFT);
+	opacity_hscl.set_tooltip_text(_("Opacity"));
+
+	// 5, brush size
+	bline_width_label.set_label(_("Brush Size:"));
+	bline_width_label.set_alignment(Gtk::ALIGN_START, Gtk::ALIGN_CENTER);
+	bline_width_label.set_sensitive(false);
+
+	bline_width_dist.set_digits(2);
+	bline_width_dist.set_range(0,10000000);
+	bline_width_dist.set_sensitive(false);
+
+	// 6, spline points
+	bline_points_label.set_label(_("Spline Points:"));
+	bline_points_label.set_alignment(Gtk::ALIGN_START, Gtk::ALIGN_CENTER);
+	bline_points_label.set_sensitive(false);
+	number_of_bline_points_spin.set_sensitive(false);
+
+	// 7, spline point angle offset
+	bline_point_angle_offset_label.set_label(_("Offset:"));
+	bline_point_angle_offset_label.set_alignment(Gtk::ALIGN_START, Gtk::ALIGN_CENTER);
+	bline_point_angle_offset_label.set_sensitive(false);
+	bline_point_angle_offset_spin.set_sensitive(false);
+
+	SPACING(bline_point_angle_offset_indent, INDENTATION);
+	bline_point_angle_offset_box.pack_start(*bline_point_angle_offset_indent, Gtk::PACK_SHRINK);
+	bline_point_angle_offset_box.pack_start(bline_point_angle_offset_label, Gtk::PACK_SHRINK);
+
+	// 8, invert
+	invert_label.set_label(_("Invert"));
+	invert_label.set_alignment(Gtk::ALIGN_START, Gtk::ALIGN_CENTER);
+
+	invert_box.pack_start(invert_label);
+	invert_box.pack_end(invert_checkbutton, Gtk::PACK_SHRINK);
+	invert_box.set_sensitive(false);
+
+	// 9, feather
+	feather_label.set_label(_("Feather:"));
+	feather_label.set_alignment(Gtk::ALIGN_START, Gtk::ALIGN_CENTER);
+	feather_label.set_sensitive(false);
+
+	feather_dist.set_digits(2);
+	feather_dist.set_range(0,10000000);
+	feather_dist.set_sensitive(false);
+
+	// 10, feather falloff for circle layer
+	falloff_label.set_label(_("Falloff:"));
+	falloff_label.set_alignment(Gtk::ALIGN_START, Gtk::ALIGN_CENTER);
+	falloff_label.set_sensitive(false);
+	SPACING(falloff_indent, INDENTATION);
+	falloff_box.pack_start(*falloff_indent, Gtk::PACK_SHRINK);
+	falloff_box.pack_start(falloff_label, Gtk::PACK_SHRINK);
+
+	falloff_enum.set_param_desc(ParamDesc("falloff")
 		.set_local_name(_("Falloff"))
 		.set_description(_("Determines the falloff function for the feather"))
 		.set_hint("enum")
@@ -449,46 +661,113 @@ StateCircle_Context::StateCircle_Context(CanvasView* canvas_view):
 		.add_enum_value(CIRCLE_SQRT,"sqrt",_("Square Root"))
 		.add_enum_value(CIRCLE_SIGMOND,"sigmond",_("Sigmond"))
 		.add_enum_value(CIRCLE_COSINE,"cosine",_("Cosine")));
+	falloff_enum.set_sensitive(false);
+
+	// 11, link origins
+	link_origins_label.set_label(_("Link Origins"));
+	link_origins_label.set_alignment(Gtk::ALIGN_START, Gtk::ALIGN_CENTER);
+
+	link_origins_box.pack_start(link_origins_label);
+	link_origins_box.pack_end(layer_link_origins_checkbutton, Gtk::PACK_SHRINK);
+	link_origins_box.set_sensitive(false);
+
+	// 12, spline origins at center
+	origins_at_center_label.set_label(_("Spline Origins at Center"));
+	origins_at_center_label.set_alignment(Gtk::ALIGN_START, Gtk::ALIGN_CENTER);
+
+	origins_at_center_box.pack_start(origins_at_center_label);
+	origins_at_center_box.pack_end(layer_origins_at_center_checkbutton, Gtk::PACK_SHRINK);
+	origins_at_center_box.set_sensitive(false);
 
-#ifdef BLEND_METHOD_IN_TOOL_OPTIONS
-	enum_blend.set_param_desc(ParamDesc(Color::BLEND_COMPOSITE,"blend_method")
-		.set_local_name(_("Blend Method"))
-		.set_description(_("Defines the blend method to be used for circles")));
-#endif	// BLEND_METHOD_IN_TOOL_OPTIONS
 
 	load_settings();
 
-	options_table.attach(*manage(new Gtk::Label(_("Circle Tool"))),		0, 2,  0,  1, Gtk::EXPAND|Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
-	options_table.attach(entry_id,										0, 2,  1,  2, Gtk::EXPAND|Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
 
-	options_table.attach(checkbutton_layer_circle, 	   	   	   	   	   	0, 2,  2,  3, Gtk::EXPAND|Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
-	options_table.attach(checkbutton_layer_outline,	   	   	   	   	   	0, 2,  3,  4, Gtk::EXPAND|Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
-	options_table.attach(checkbutton_layer_advanced_outline,  	   	   	0, 2,  4,  5, Gtk::EXPAND|Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
-	options_table.attach(checkbutton_layer_region,						0, 2,  5,  6, Gtk::EXPAND|Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
-	options_table.attach(checkbutton_layer_plant,						0, 2,  6,  7, Gtk::EXPAND|Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
-	options_table.attach(checkbutton_layer_curve_gradient,				0, 2,  7,  8, Gtk::EXPAND|Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
-	options_table.attach(checkbutton_layer_link_origins,				0, 2,  8,  9, Gtk::EXPAND|Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
-	options_table.attach(checkbutton_layer_origins_at_center,			0, 2,  9,  10, Gtk::EXPAND|Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
+	// pack all options to the options_table
 
-	//invert flag
-	options_table.attach(checkbutton_invert,					    	0, 2,  10, 11, Gtk::EXPAND|Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
+	// 0, title
+	options_table.attach(title_label,
+		0, 2, 0, 1, Gtk::FILL, Gtk::FILL, 0, 0
+		);
+	// 1, name
+	options_table.attach(id_box,
+		0, 2, 1, 2, Gtk::FILL, Gtk::FILL, 0, 0
+		);
+	// 2, layer types creation
+	options_table.attach(layer_types_label,
+		0, 2, 2, 3, Gtk::FILL, Gtk::FILL, 0, 0
+		);
+	options_table.attach(layer_types_box,
+		0, 2, 3, 4, Gtk::FILL, Gtk::FILL, 0, 0
+		);
+	// 3, blend method
+	options_table.attach(blend_box,
+		0, 1, 4, 5, Gtk::EXPAND|Gtk::FILL, Gtk::FILL, 0, 0
+		);
+	options_table.attach(blend_enum,
+		1, 2, 4, 5, Gtk::EXPAND|Gtk::FILL, Gtk::FILL, 0, 0
+		);
+	// 4, opacity
+	options_table.attach(opacity_label,
+		0, 1, 5, 6, Gtk::EXPAND|Gtk::FILL, Gtk::FILL, 0, 0
+		);
+	options_table.attach(opacity_hscl,
+		1, 2, 5, 6, Gtk::EXPAND|Gtk::FILL, Gtk::FILL, 0, 0
+		);
+	// 5, brush size
+	options_table.attach(bline_width_label,
+		0, 1, 6, 7, Gtk::EXPAND|Gtk::FILL, Gtk::FILL, 0, 0
+		);
+	options_table.attach(bline_width_dist,
+		1, 2, 6, 7, Gtk::EXPAND|Gtk::FILL, Gtk::FILL, 0, 0
+		);
+	// 6, spline points
+	options_table.attach(bline_points_label,
+		0, 1, 7, 8, Gtk::EXPAND|Gtk::FILL, Gtk::FILL, 0, 0
+		);
+	options_table.attach(number_of_bline_points_spin,
+		1, 2, 7, 8, Gtk::EXPAND|Gtk::FILL, Gtk::FILL, 0, 0
+		);
+	// 7, spline points offset
+	options_table.attach(bline_point_angle_offset_box,
+		0, 1, 8, 9, Gtk::EXPAND|Gtk::FILL, Gtk::FILL, 0, 0
+		);
+	options_table.attach(bline_point_angle_offset_spin,
+		1, 2, 8, 9, Gtk::EXPAND|Gtk::FILL, Gtk::FILL, 0, 0
+		);
+	// 8, invert
+	options_table.attach(invert_box,
+		0, 2, 9, 10, Gtk::EXPAND|Gtk::FILL, Gtk::FILL, 0, 0
+		);
+	// 9, feather
+	options_table.attach(feather_label,
+		0, 1, 10, 11, Gtk::EXPAND|Gtk::FILL, Gtk::FILL, 0, 0
+		);
+	options_table.attach(feather_dist,
+		1, 2, 10, 11, Gtk::EXPAND|Gtk::FILL, Gtk::FILL, 0, 0
+		);
+  // 10, falloff
+  options_table.attach(falloff_box,
+		0, 1, 11, 12, Gtk::EXPAND|Gtk::FILL, Gtk::FILL, 0, 0
+		);
+	options_table.attach(falloff_enum,
+		1, 2, 11, 12, Gtk::EXPAND|Gtk::FILL, Gtk::FILL, 0, 0
+		);
+	// 11, link origins
+	options_table.attach(link_origins_box,
+		0, 2, 12, 13, Gtk::FILL, Gtk::FILL, 0, 0
+		);
+	// 12, origins at center
+	options_table.attach(origins_at_center_box,
+		0, 2, 13, 14, Gtk::FILL, Gtk::FILL, 0, 0
+		);
 
-	options_table.attach(*manage(new Gtk::Label(_("Falloff:"))),		0, 1, 11, 12, Gtk::EXPAND|Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
-	options_table.attach(enum_falloff,                              	1, 2, 11, 12, Gtk::EXPAND|Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
-
-	//feather stuff
-	options_table.attach(*manage(new Gtk::Label(_("Feather:"))),    	0, 1, 12, 13, Gtk::EXPAND|Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
-   	options_table.attach(spin_feather, 							    	1, 2, 12, 14, Gtk::EXPAND|Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
-
-#ifdef BLEND_METHOD_IN_TOOL_OPTIONS
-	options_table.attach(enum_blend,                                	0, 2, 13, 14, Gtk::EXPAND|Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
-#endif	// BLEND_METHOD_IN_TOOL_OPTIONS
-
-	options_table.attach(*manage(new Gtk::Label(_("Spline Points:"))),	0, 1, 14, 15, Gtk::EXPAND|Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
-	options_table.attach(spin_number_of_bline_points,					1, 2, 14, 15, Gtk::EXPAND|Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
-
-options_table.attach(*manage(new Gtk::Label(_("Point Angle Offset:"))),	0, 1, 15, 16, Gtk::EXPAND|Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
-	options_table.attach(spin_bline_point_angle_offset,					1, 2, 15, 16, Gtk::EXPAND|Gtk::FILL, Gtk::EXPAND|Gtk::FILL, 0, 0);
+	// fine-tune options layout
+	options_table.set_border_width(GAP*2); // border width
+	options_table.set_row_spacings(GAP); // row gap
+	options_table.set_row_spacing(0, GAP*2); // the gap between first and second row.
+	options_table.set_row_spacing(2, 1); // row gap between label and icon of layer type
+	options_table.set_row_spacing(14, 0); // the final row using border width of table
 
 	options_table.show_all();
 
@@ -506,7 +785,7 @@ options_table.attach(*manage(new Gtk::Label(_("Point Angle Offset:"))),	0, 1, 15
 
 	get_work_area()->set_cursor(Gdk::CROSSHAIR);
 
-	App::toolbox->refresh();
+	App::dock_toolbox->refresh();
 }
 
 void
@@ -540,7 +819,7 @@ StateCircle_Context::~StateCircle_Context()
 
 	get_canvas_view()->queue_rebuild_ducks();
 
-	App::toolbox->refresh();
+	App::dock_toolbox->refresh();
 }
 
 Smach::event_result
@@ -640,7 +919,9 @@ StateCircle_Context::make_circle(const Point& _p1, const Point& _p2)
 	if (get_layer_circle_flag() &&
 		get_falloff() >= 0 && get_falloff() < CIRCLE_NUM_FALLOFF)
 	{
+		egress_on_selection_change=false;
 		layer=get_canvas_interface()->add_layer_to("circle",canvas,depth);
+		egress_on_selection_change=true;
 		if (!layer)
 		{
 			get_canvas_view()->get_ui_interface()->error(_("Unable to create layer"));
@@ -655,16 +936,17 @@ StateCircle_Context::make_circle(const Point& _p1, const Point& _p2)
 		layer->set_param("falloff",get_falloff());
 		get_canvas_interface()->signal_layer_param_changed()(layer,"falloff");
 
-		layer->set_param("feather",get_feather());
+		layer->set_param("amount",get_opacity());
+		get_canvas_interface()->signal_layer_param_changed()(layer,"amount");
+
+		layer->set_param("feather",get_feather_size());
 		get_canvas_interface()->signal_layer_param_changed()(layer,"feather");
 
 		layer->set_param("invert",get_invert());
 		get_canvas_interface()->signal_layer_param_changed()(layer,"invert");
 
-#ifdef BLEND_METHOD_IN_TOOL_OPTIONS
 		layer->set_param("blend_method",get_blend());
 		get_canvas_interface()->signal_layer_param_changed()(layer,"blend_method");
-#endif	// BLEND_METHOD_IN_TOOL_OPTIONS
 
 		layer->set_description(get_id());
 		get_canvas_interface()->signal_layer_new_description()(layer,layer->get_description());
@@ -706,7 +988,9 @@ StateCircle_Context::make_circle(const Point& _p1, const Point& _p2)
 	{
 		synfigapp::PushMode push_mode(get_canvas_interface(),synfigapp::MODE_NORMAL);
 
+		egress_on_selection_change=false;
 		Layer::Handle layer(get_canvas_interface()->add_layer_to("curve_gradient",canvas,depth));
+		egress_on_selection_change=true;
 		if (!layer)
 		{
 			get_canvas_view()->get_ui_interface()->error(_("Unable to create layer"));
@@ -717,10 +1001,14 @@ StateCircle_Context::make_circle(const Point& _p1, const Point& _p2)
 		layer->set_description(get_id()+_(" Gradient"));
 		get_canvas_interface()->signal_layer_new_description()(layer,layer->get_description());
 
-#ifdef BLEND_METHOD_IN_TOOL_OPTIONS
 		layer->set_param("blend_method",get_blend());
 		get_canvas_interface()->signal_layer_param_changed()(layer,"blend_method");
-#endif	// BLEND_METHOD_IN_TOOL_OPTIONS
+
+		layer->set_param("amount",get_opacity());
+		get_canvas_interface()->signal_layer_param_changed()(layer,"amount");
+
+		layer->set_param("width",get_bline_width());
+		get_canvas_interface()->signal_layer_param_changed()(layer,"width");
 
 		{
 			synfigapp::Action::Handle action(synfigapp::Action::create("LayerParamConnect"));
@@ -778,7 +1066,9 @@ StateCircle_Context::make_circle(const Point& _p1, const Point& _p2)
 	{
 		synfigapp::PushMode push_mode(get_canvas_interface(),synfigapp::MODE_NORMAL);
 
+		egress_on_selection_change=false;
 		Layer::Handle layer(get_canvas_interface()->add_layer_to("plant",canvas,depth));
+		egress_on_selection_change=true;
 		if (!layer)
 		{
 			get_canvas_view()->get_ui_interface()->error(_("Unable to create layer"));
@@ -789,10 +1079,11 @@ StateCircle_Context::make_circle(const Point& _p1, const Point& _p2)
 		layer->set_description(get_id()+_(" Plant"));
 		get_canvas_interface()->signal_layer_new_description()(layer,layer->get_description());
 
-#ifdef BLEND_METHOD_IN_TOOL_OPTIONS
 		layer->set_param("blend_method",get_blend());
 		get_canvas_interface()->signal_layer_param_changed()(layer,"blend_method");
-#endif	// BLEND_METHOD_IN_TOOL_OPTIONS
+
+		layer->set_param("amount",get_opacity());
+		get_canvas_interface()->signal_layer_param_changed()(layer,"amount");
 
 		{
 			synfigapp::Action::Handle action(synfigapp::Action::create("LayerParamConnect"));
@@ -850,7 +1141,9 @@ StateCircle_Context::make_circle(const Point& _p1, const Point& _p2)
 	{
 		synfigapp::PushMode push_mode(get_canvas_interface(),synfigapp::MODE_NORMAL);
 
+		egress_on_selection_change=false;
 		Layer::Handle layer(get_canvas_interface()->add_layer_to("region",canvas,depth));
+		egress_on_selection_change=true;
 		if (!layer)
 		{
 			get_canvas_view()->get_ui_interface()->error(_("Unable to create layer"));
@@ -861,12 +1154,13 @@ StateCircle_Context::make_circle(const Point& _p1, const Point& _p2)
 		layer->set_description(get_id()+_(" Region"));
 		get_canvas_interface()->signal_layer_new_description()(layer,layer->get_description());
 
-#ifdef BLEND_METHOD_IN_TOOL_OPTIONS
 		layer->set_param("blend_method",get_blend());
 		get_canvas_interface()->signal_layer_param_changed()(layer,"blend_method");
-#endif	// BLEND_METHOD_IN_TOOL_OPTIONS
 
-		layer->set_param("feather",get_feather());
+		layer->set_param("amount",get_opacity());
+		get_canvas_interface()->signal_layer_param_changed()(layer,"amount");
+
+		layer->set_param("feather",get_feather_size());
 		get_canvas_interface()->signal_layer_param_changed()(layer,"feather");
 
 		layer->set_param("invert",get_invert());
@@ -930,7 +1224,9 @@ StateCircle_Context::make_circle(const Point& _p1, const Point& _p2)
 
 	if (get_layer_outline_flag())
 	{
+		egress_on_selection_change=false;
 		Layer::Handle layer(get_canvas_interface()->add_layer_to("outline",canvas,depth));
+		egress_on_selection_change=true;
 		if (!layer)
 		{
 			get_canvas_view()->get_ui_interface()->error(_("Unable to create layer"));
@@ -941,12 +1237,16 @@ StateCircle_Context::make_circle(const Point& _p1, const Point& _p2)
 		layer->set_description(get_id()+_(" Outline"));
 		get_canvas_interface()->signal_layer_new_description()(layer,layer->get_description());
 
-#ifdef BLEND_METHOD_IN_TOOL_OPTIONS
 		layer->set_param("blend_method",get_blend());
 		get_canvas_interface()->signal_layer_param_changed()(layer,"blend_method");
-#endif	// BLEND_METHOD_IN_TOOL_OPTIONS
 
-		layer->set_param("feather",get_feather());
+		layer->set_param("amount",get_opacity());
+		get_canvas_interface()->signal_layer_param_changed()(layer,"amount");
+
+		layer->set_param("width",get_bline_width());
+		get_canvas_interface()->signal_layer_param_changed()(layer,"width");
+
+		layer->set_param("feather",get_feather_size());
 		get_canvas_interface()->signal_layer_param_changed()(layer,"feather");
 
 		layer->set_param("invert",get_invert());
@@ -1007,7 +1307,9 @@ StateCircle_Context::make_circle(const Point& _p1, const Point& _p2)
 	if (get_layer_advanced_outline_flag())
 	{
 		synfigapp::PushMode push_mode(get_canvas_interface(),synfigapp::MODE_NORMAL);
+		egress_on_selection_change=false;
 		Layer::Handle layer(get_canvas_interface()->add_layer_to("advanced_outline",canvas,depth));
+		egress_on_selection_change=true;
 		if (!layer)
 		{
 			get_canvas_view()->get_ui_interface()->error(_("Unable to create layer"));
@@ -1018,12 +1320,16 @@ StateCircle_Context::make_circle(const Point& _p1, const Point& _p2)
 		layer->set_description(get_id()+_(" Advanced Outline"));
 		get_canvas_interface()->signal_layer_new_description()(layer,layer->get_description());
 
-#ifdef BLEND_METHOD_IN_TOOL_OPTIONS
 		layer->set_param("blend_method",get_blend());
 		get_canvas_interface()->signal_layer_param_changed()(layer,"blend_method");
-#endif	// BLEND_METHOD_IN_TOOL_OPTIONS
 
-		layer->set_param("feather",get_feather());
+		layer->set_param("amount",get_opacity());
+		get_canvas_interface()->signal_layer_param_changed()(layer,"amount");
+
+		layer->set_param("width",get_bline_width());
+		get_canvas_interface()->signal_layer_param_changed()(layer,"width");
+
+		layer->set_param("feather",get_feather_size());
 		get_canvas_interface()->signal_layer_param_changed()(layer,"feather");
 
 		layer->set_param("invert",get_invert());
@@ -1132,7 +1438,8 @@ StateCircle_Context::event_mouse_click_handler(const Smach::event& x)
 			if ((point[1] - point_holder[1]) < 0) point[1] = point_holder[1];
 		}
 
-		make_circle(point_holder, point);
+		if (point_holder != point)
+			make_circle(point_holder, point);
 		get_work_area()->clear_ducks();
 		return Smach::RESULT_ACCEPT;
 	}
@@ -1147,3 +1454,132 @@ StateCircle_Context::refresh_ducks()
 	get_work_area()->clear_ducks();
 	get_work_area()->queue_draw();
 }
+
+
+void
+StateCircle_Context::toggle_layer_creation()
+{
+  // don't allow none layer creation
+  if (get_layer_circle_flag() +
+     get_layer_region_flag() +
+     get_layer_outline_flag() +
+     get_layer_advanced_outline_flag() +
+     get_layer_curve_gradient_flag() +
+     get_layer_plant_flag() == 0)
+  {
+    if(layer_circle_flag) set_layer_circle_flag(true);
+    else if(layer_region_flag) set_layer_region_flag(true);
+    else if(layer_outline_flag) set_layer_outline_flag(true);
+    else if(layer_advanced_outline_flag) set_layer_advanced_outline_flag(true);
+    else if(layer_curve_gradient_flag) set_layer_curve_gradient_flag(true);
+    else if(layer_plant_flag) set_layer_plant_flag(true);
+  }
+
+	// brush size
+	if (get_layer_outline_flag() ||
+		get_layer_advanced_outline_flag() ||
+		get_layer_curve_gradient_flag())
+	{
+		bline_width_label.set_sensitive(true);
+		bline_width_dist.set_sensitive(true);
+	}
+	else
+	{
+		bline_width_label.set_sensitive(false);
+		bline_width_dist.set_sensitive(false);
+	}
+
+	// spline points and offset angle
+	if (!get_layer_region_flag() &&
+		!get_layer_outline_flag() &&
+		!get_layer_advanced_outline_flag() &&
+		!get_layer_plant_flag() &&
+		!get_layer_curve_gradient_flag())
+	{
+		bline_points_label.set_sensitive(false);
+		number_of_bline_points_spin.set_sensitive(false);
+		bline_point_angle_offset_label.set_sensitive(false);
+		bline_point_angle_offset_spin.set_sensitive(false);
+	}
+	else
+	{
+		bline_points_label.set_sensitive(true);
+		number_of_bline_points_spin.set_sensitive(true);
+		bline_point_angle_offset_label.set_sensitive(true);
+		bline_point_angle_offset_spin.set_sensitive(true);
+	}
+
+	// invert
+	if (get_layer_circle_flag() ||
+		get_layer_region_flag() ||
+		get_layer_outline_flag() ||
+		get_layer_advanced_outline_flag())
+	{
+		invert_box.set_sensitive(true);
+	}
+	else
+		invert_box.set_sensitive(false);
+
+	// feather size
+	if (get_layer_circle_flag() ||
+		get_layer_circle_flag() ||
+		get_layer_region_flag() ||
+		get_layer_outline_flag() ||
+		get_layer_advanced_outline_flag())
+	{
+		feather_label.set_sensitive(true);
+		feather_dist.set_sensitive(true);
+	}
+	else
+	{
+		feather_label.set_sensitive(false);
+		feather_dist.set_sensitive(false);
+	}
+
+	// falloff type for circle layer only
+	if (get_layer_circle_flag())
+	{
+		feather_dist.set_sensitive(true);
+		feather_label.set_sensitive(true);
+
+		falloff_label.set_sensitive(true);
+		falloff_enum.set_sensitive(true);
+	}
+	else
+	{
+		falloff_label.set_sensitive(false);
+		falloff_enum.set_sensitive(false);
+	}
+
+	// orignis at center
+	if (get_layer_region_flag() ||
+		get_layer_outline_flag() ||
+		get_layer_advanced_outline_flag() ||
+		get_layer_plant_flag() ||
+		get_layer_curve_gradient_flag())
+	{
+		origins_at_center_box.set_sensitive(true);
+	}
+	else
+		origins_at_center_box.set_sensitive(false);
+
+	// link origins
+	if (get_layer_region_flag() +
+		get_layer_outline_flag() +
+		get_layer_advanced_outline_flag() +
+		get_layer_plant_flag() +
+		get_layer_curve_gradient_flag() +
+		get_layer_circle_flag() >= 2)
+		{
+			link_origins_box.set_sensitive(true);
+		}
+	else link_origins_box.set_sensitive(false);
+
+  // update layer flags
+  layer_circle_flag = get_layer_circle_flag();
+  layer_region_flag = get_layer_region_flag();
+  layer_outline_flag = get_layer_outline_flag();
+  layer_advanced_outline_flag = get_layer_advanced_outline_flag();
+  layer_curve_gradient_flag = get_layer_curve_gradient_flag();
+  layer_plant_flag = get_layer_plant_flag();
+}
