@@ -41,7 +41,7 @@
 #include <synfigapp/value_desc.h>
 #include "layertreestore.h"
 #include "layerparamtreestore.h"
-#include <synfig/valuenode_animated.h>
+#include <synfig/valuenodes/valuenode_animated.h>
 
 #include "widgets/widget_value.h"
 
@@ -108,7 +108,7 @@ private:
 
 	Gtk::HBox *hbox;
 
-	Gtk::Adjustment layer_amount_adjustment_;
+	Glib::RefPtr<Gtk::Adjustment> layer_amount_adjustment_;
 
 	Gtk::HScale *layer_amount_hscale;
 
@@ -201,14 +201,8 @@ private:
 
 	void on_blend_method_changed();
 
-	/* GTKMM 3
-	void on_param_column_label_tree_style_updated ();
-	*/
-	void on_param_tree_column_label_style_changed (const Glib::RefPtr< Gtk::Style >& previous_style);
-	/* GTKMM 3
-	bool on_param_column_label_tree_draw (const ::Cairo::RefPtr< ::Cairo::Context>& cr);
-	*/
-	bool on_param_tree_column_label_expose_draw (GdkEventExpose * event);
+	void on_param_column_label_tree_style_updated();
+	bool on_param_column_label_tree_draw(const ::Cairo::RefPtr< ::Cairo::Context>& cr);
 
 public:
 
@@ -246,7 +240,7 @@ public:
 
 	void set_model(Glib::RefPtr<LayerTreeStore> layer_tree_store_);
 
-	void set_time_adjustment(Gtk::Adjustment &adjustment);
+	void set_time_adjustment(const Glib::RefPtr<Gtk::Adjustment> &adjustment);
 
 	void set_show_timetrack(bool x=true);
 
