@@ -60,9 +60,9 @@ ACTION_SET_CVS_ID(Action::LayerParamSetStatic,"$Id$");
 
 /* === M E T H O D S ======================================================= */
 
-Action::LayerParamSetStatic::LayerParamSetStatic()
-{
-}
+Action::LayerParamSetStatic::LayerParamSetStatic():
+	old_static_value()
+{ }
 
 Action::ParamVocab
 Action::LayerParamSetStatic::get_param_vocab()
@@ -84,7 +84,7 @@ Action::LayerParamSetStatic::is_candidate(const ParamList &x)
 
 	ValueDesc value_desc(x.find("value_desc")->second.get_value_desc());
 
-	if(!value_desc.parent_is_layer_param())
+	if(!value_desc.parent_is_layer())
 		return false;
 
 	synfig::ValueBase parameter;
@@ -102,7 +102,7 @@ Action::LayerParamSetStatic::is_candidate(const ParamList &x)
 	//! Retrieves the current parameter
 	parameter = _layer->get_param(_param_name);
 	//! Check that the parameter is not a inline canvas
-	if(parameter.get_type()==ValueBase::TYPE_CANVAS && parameter.get(Canvas::Handle()))
+	if(parameter.get_type()==type_canvas && parameter.get(Canvas::Handle()))
 		if(parameter.get(Canvas::Handle())->is_inline())
 			return false;
 	//! Check if it is already static
@@ -120,7 +120,7 @@ Action::LayerParamSetStatic::set_param(const synfig::String& name, const Action:
 	if(!layer && name=="value_desc" && param.get_type()==Param::TYPE_VALUEDESC)
 	{
 		ValueDesc value_desc(param.get_value_desc());
-		if(!value_desc.parent_is_layer_param())
+		if(!value_desc.parent_is_layer())
 			return false;
 
 		layer=Layer::Handle::cast_dynamic(value_desc.get_layer());
@@ -133,7 +133,7 @@ Action::LayerParamSetStatic::set_param(const synfig::String& name, const Action:
 	if(param_name.empty() && name=="value_desc" && param.get_type()==Param::TYPE_VALUEDESC)
 	{
 		ValueDesc value_desc(param.get_value_desc());
-		if(!value_desc.parent_is_layer_param())
+		if(!value_desc.parent_is_layer())
 			return false;
 
 		param_name=value_desc.get_param_name();
