@@ -34,11 +34,12 @@
 #include "valuedesclink.h"
 
 #include <synfigapp/canvasinterface.h>
-#include <synfig/valuenode_const.h>
-#include <synfig/valuenode_scale.h>
-#include <synfig/valuenode_composite.h>
+#include <synfig/valuenodes/valuenode_const.h>
+#include <synfig/valuenodes/valuenode_scale.h>
+#include <synfig/valuenodes/valuenode_composite.h>
 
 #include <synfigapp/general.h>
+#include <zconf.h>
 
 #endif
 
@@ -117,15 +118,35 @@ Action::ValueDescLink::set_param(const synfig::String& name, const Action::Param
 		if(value_desc.is_value_node() && value_desc.parent_is_linkable_value_node())
 		{
 			synfig::ValueNode_Composite::Handle compo(synfig::ValueNode_Composite::Handle::cast_dynamic(value_desc.get_value_node()));
-			if(compo && compo->get_type() == ValueBase::TYPE_WIDTHPOINT)
+			if(compo)
 			{
-				synfigapp::Action::Param param(synfigapp::ValueDesc(compo, compo->get_link_index_from_name("position")));
-				return set_param("value_desc", param);
-			}
-			if(compo && compo->get_type() == ValueBase::TYPE_BLINEPOINT)
-			{
-				synfigapp::Action::Param param(synfigapp::ValueDesc(compo, compo->get_link_index_from_name("point")));
-				return set_param("value_desc", param);
+				String param_name;
+				if (!value_desc.get_sub_name().empty())
+				{
+					LinkableValueNode::Vocab vocab = compo->get_children_vocab();
+					for(LinkableValueNode::Vocab::const_iterator i = vocab.begin(); i != vocab.end(); ++i)
+						if (i->get_name() == value_desc.get_sub_name())
+							param_name = value_desc.get_sub_name();
+				}
+				
+				//! \todo these check may be unused 
+				if (param_name.empty() && compo->get_type() == type_width_point)
+					param_name = "position";
+				if (param_name.empty() && compo->get_type() == type_bline_point)
+					param_name = "point";
+				
+				if ( compo->get_type() == type_bline_point
+				  && param_name == "t2"
+				  && (*compo)(time).get(BLinePoint()).get_merge_tangent_both())
+				{
+					param_name = "t1";
+				}
+				
+				if (!param_name.empty())
+				{
+					synfigapp::Action::Param param(synfigapp::ValueDesc(compo, compo->get_link_index_from_name(param_name)));
+					return set_param("value_desc", param);
+				}
 			}
 		}
 
@@ -261,8 +282,8 @@ Action::ValueDescLink::set_param(const synfig::String& name, const Action::Param
 			// Everything must be of the same type
 			poison=true;
 			status_message = (strprintf(_("Cannot link two values of different types ('%s' and '%s')"),
-										ValueBase::type_local_name(value_desc.get_value_type()).c_str(),
-										ValueBase::type_local_name(value_desc_list.front().get_value_type()).c_str()));
+										value_desc.get_value_type().description.local_name.c_str(),
+										value_desc_list.front().get_value_type().description.local_name.c_str()));
 			return false;
 		}
 
