@@ -36,22 +36,30 @@
 #include "valuenodeconstset.h"
 #include "valuedescconnect.h"
 #include "waypointsetsmart.h"
+#include "waypointset.h"
 
 #include "valuedescset.h"
 #include <synfigapp/canvasinterface.h>
-#include <synfig/valuenode_bline.h>
-#include <synfig/valuenode_wplist.h>
-#include <synfig/valuenode_blinecalctangent.h>
-#include <synfig/valuenode_blinecalcvertex.h>
-#include <synfig/valuenode_blinecalcwidth.h>
-#include <synfig/valuenode_composite.h>
-#include <synfig/valuenode_radialcomposite.h>
-#include <synfig/valuenode_range.h>
-#include <synfig/valuenode_reference.h>
-#include <synfig/valuenode_boneinfluence.h>
-#include <synfig/valuenode_scale.h>
-#include <synfig/valuenode_integer.h>
-#include <synfig/valuenode_real.h>
+#include <synfig/valuenodes/valuenode_bline.h>
+#include <synfig/valuenodes/valuenode_wplist.h>
+#include <synfig/valuenodes/valuenode_blinecalctangent.h>
+#include <synfig/valuenodes/valuenode_blinecalcvertex.h>
+#include <synfig/valuenodes/valuenode_blinecalcwidth.h>
+#include <synfig/valuenodes/valuenode_composite.h>
+#include <synfig/valuenodes/valuenode_radialcomposite.h>
+#include <synfig/valuenodes/valuenode_range.h>
+#include <synfig/valuenodes/valuenode_reference.h>
+#include <synfig/valuenodes/valuenode_boneinfluence.h>
+#include <synfig/valuenodes/valuenode_scale.h>
+#include <synfig/valuenodes/valuenode_integer.h>
+#include <synfig/valuenodes/valuenode_real.h>
+#include <synfig/valuenodes/valuenode_bonelink.h>
+#include <synfig/valuenodes/valuenode_bone.h>
+#include <synfig/valuenodes/valuenode_average.h>
+#include <synfig/valuenodes/valuenode_weightedaverage.h>
+#include <synfig/valueoperations.h>
+#include <synfig/weightedvalue.h>
+#include <synfig/pair.h>
 #include <synfigapp/main.h>
 
 #include <synfigapp/general.h>
@@ -82,7 +90,10 @@ ACTION_SET_CVS_ID(Action::ValueDescSet,"$Id$");
 /* === M E T H O D S ======================================================= */
 
 Action::ValueDescSet::ValueDescSet():
-	time(0)
+	time(0),
+	recursive(true),
+	animate(false),
+	lock_animation(false)
 {
 }
 
@@ -112,6 +123,18 @@ Action::ValueDescSet::get_param_vocab()
 		.set_local_name(_("Time"))
 		.set_optional()
 	);
+	ret.push_back(ParamDesc("recursive", Param::TYPE_BOOL)
+		.set_local_name(_("Recursive"))
+		.set_optional()
+	);
+	ret.push_back(ParamDesc("animate", Param::TYPE_BOOL)
+		.set_local_name(_("Animate"))
+		.set_optional()
+	);
+	ret.push_back(ParamDesc("lock_animation", Param::TYPE_BOOL)
+		.set_local_name(_("Lock animation"))
+		.set_optional()
+	);
 
 	return ret;
 }
@@ -128,21 +151,31 @@ Action::ValueDescSet::set_param(const synfig::String& name, const Action::Param
 	if(name=="value_desc" && param.get_type()==Param::TYPE_VALUEDESC)
 	{
 		value_desc=param.get_value_desc();
-
 		return true;
 	}
-
 	if(name=="new_value" && param.get_type()==Param::TYPE_VALUE)
 	{
 		value=param.get_value();
-
 		return true;
 	}
-
 	if(name=="time" && param.get_type()==Param::TYPE_TIME)
 	{
 		time=param.get_time();
-
+		return true;
+	}
+	if(name=="recursive" && param.get_type()==Param::TYPE_BOOL)
+	{
+		recursive=param.get_bool();
+		return true;
+	}
+	if(name=="animate" && param.get_type()==Param::TYPE_BOOL)
+	{
+		animate=param.get_bool();
+		return true;
+	}
+	if(name=="lock_animation" && param.get_type()==Param::TYPE_BOOL)
+	{
+		lock_animation=param.get_bool();
 		return true;
 	}
 
@@ -162,32 +195,6 @@ Action::ValueDescSet::prepare()
 {
 	clear();
 
-	// If our tangents are merged, and
-	// our first tangent is being manipulated,
-	// then we also need to adjust the other
-	// tangent.
-	if(	value_desc.parent_is_value_node() &&
-		value_desc.get_parent_value_node()->get_type()==ValueBase::TYPE_BLINEPOINT &&
-		(value_desc.get_name()=="t1" || value_desc.get_name()=="t2") &&
-		(*value_desc.get_parent_value_node())(time).get(BLinePoint()).get_split_tangent_flag()==false)
-	{
-		if (value_desc.get_name()=="t1") {
-			ValueNode_Composite::Handle parent_value_node;
-			parent_value_node=parent_value_node.cast_dynamic(value_desc.get_parent_value_node());
-			assert(parent_value_node);
-			Action::Handle action(Action::create("ValueDescSet"));
-			if(!action)
-				throw Error(_("Unable to find action ValueDescSet (bug)"));
-			action->set_param("canvas",get_canvas());
-			action->set_param("canvas_interface",get_canvas_interface());
-			action->set_param("time",time);
-			action->set_param("new_value",value);
-			action->set_param("value_desc",ValueDesc(parent_value_node, parent_value_node->get_link_index_from_name("t2")));
-			if(!action->is_ready())
-				throw Error(Error::TYPE_NOTREADY);
-			add_action(action);
-		}
-	}
 	// If we are a reference value node, then
 	// we need to distribute the changes to the
 	// referenced value node
@@ -209,7 +216,9 @@ Action::ValueDescSet::prepare()
 	}
 
 	// if we are a boneinfluence value node, then we need to distribute the changes to the linked value node
-	if(value_desc.is_value_node())
+	// This is only valid for vector type converted to bone influence
+	// Not valid for a whole blinepoint converted to bone influence
+	if(value_desc.is_value_node() && value.get_type() == type_vector)
 	{
 		if (ValueNode_BoneInfluence::Handle bone_influence_value_node =
 			ValueNode_BoneInfluence::Handle::cast_dynamic(value_desc.get_value_node()))
@@ -218,7 +227,7 @@ Action::ValueDescSet::prepare()
 												bone_influence_value_node->get_link_index_from_name("link"));
 
 			if (bone_influence_value_node->has_inverse_transform())
-				value = bone_influence_value_node->get_inverse_transform().get_transformed(value);
+				value = bone_influence_value_node->get_inverse_transform().get_transformed(value.get(Vector()));
 			else
 				throw Error(_("this node isn't editable - in the future it will be greyed to prevent editing"));
 
@@ -242,62 +251,296 @@ Action::ValueDescSet::prepare()
 		}
 	}
 
+    // Set ValueNode_Average
+	if(value_desc.is_value_node() && ValueNode_Average::check_type(value.get_type()))
+	{
+		if (ValueNode_Average::Handle bone_average_value_node =
+			ValueNode_Average::Handle::cast_dynamic(value_desc.get_value_node()))
+		{
+			ValueBase::List values_list;
+			values_list.reserve(bone_average_value_node->link_count());
+			for(int i = 0; i < bone_average_value_node->link_count(); ++i)
+				values_list.push_back((*bone_average_value_node->get_link(i))(time));
+
+			ValueAverage::set_average_value_generic(values_list.begin(), values_list.end(), value);
+
+			for(int i = 0; i < bone_average_value_node->link_count(); ++i)
+			{
+				Action::Handle action(Action::create("ValueDescSet"));
+
+				if(!action)
+					throw Error(_("Unable to find action ValueDescSet (bug)"));
+
+				action->set_param("canvas",get_canvas());
+				action->set_param("canvas_interface",get_canvas_interface());
+				action->set_param("time",time);
+				action->set_param("new_value",values_list[i]);
+				action->set_param("value_desc",ValueDesc(value_desc.get_value_node(), i));
+
+				if(!action->is_ready())
+					throw Error(Error::TYPE_NOTREADY);
+
+				add_action(action);
+			}
+
+			return;
+		}
+	}
+
+    // Set ValueNode_WeightedAverage
+	if(value_desc.is_value_node() && ValueNode_WeightedAverage::check_type(value.get_type()))
+	{
+		if (ValueNode_WeightedAverage::Handle bone_weighted_average_value_node =
+			ValueNode_WeightedAverage::Handle::cast_dynamic(value_desc.get_value_node()))
+		{
+			ValueBase::List values_list;
+			values_list.reserve(bone_weighted_average_value_node->link_count());
+			for(int i = 0; i < bone_weighted_average_value_node->link_count(); ++i)
+				values_list.push_back((*bone_weighted_average_value_node->get_link(i))(time));
+
+			ValueBase values_list_value(values_list);
+			ValueAverage::set_average_value_weighted(values_list_value, value);
+			const ValueBase::List &new_values_list = values_list_value.get_list();
+
+			for(int i = 0; i < bone_weighted_average_value_node->link_count(); ++i)
+			{
+				Action::Handle action(Action::create("ValueDescSet"));
+
+				if(!action)
+					throw Error(_("Unable to find action ValueDescSet (bug)"));
+
+				action->set_param("canvas",get_canvas());
+				action->set_param("canvas_interface",get_canvas_interface());
+				action->set_param("time",time);
+				action->set_param("new_value",new_values_list[i]);
+				action->set_param("value_desc",ValueDesc(value_desc.get_value_node(), i));
+
+				if(!action->is_ready())
+					throw Error(Error::TYPE_NOTREADY);
+
+				add_action(action);
+			}
+
+			return;
+		}
+	}
+
+	// If we are a bone link value node, then
+	// we need to change the transformation part
+	if(value_desc.is_value_node() && ValueNode_BoneLink::Handle::cast_dynamic(value_desc.get_value_node()))
+	{
+		ValueNode_BoneLink::Handle value_node = ValueNode_BoneLink::Handle::cast_dynamic(value_desc.get_value_node());
+		ValueDesc base_value_desc(value_node, value_node->get_link_index_from_name("base_value"));
+
+		ValueBase new_base_value =
+			ValueTransformation::back_transform(
+				value_node->get_bone_transformation(time), value );
+
+		Action::Handle action(Action::create("ValueDescSet"));
+		if(!action)
+			throw Error(_("Unable to find action ValueDescSet (bug)"));
+		action->set_param("canvas",get_canvas());
+		action->set_param("canvas_interface",get_canvas_interface());
+		action->set_param("time",time);
+		action->set_param("new_value",new_base_value);
+		action->set_param("value_desc",base_value_desc);
+		if(!action->is_ready())
+			throw Error(Error::TYPE_NOTREADY);
+		add_action(action);
+		return;
+	}
+
 	// If we are a composite value node, then
 	// we need to distribute the changes to the
 	// individual parts
 	// except if we are TYPE WIDTHPOINT which is handled individually
-	if(value_desc.is_value_node() && ValueNode_Composite::Handle::cast_dynamic(value_desc.get_value_node())
-	 && value_desc.get_value_node()->get_type()!=ValueBase::TYPE_WIDTHPOINT)
+	// except if we are TYPE BLINEPOINT which is handled individually
+	if (value_desc.is_value_node()
+	 && ValueNode_Composite::Handle::cast_dynamic(value_desc.get_value_node())
+	 && value_desc.get_value_node()->get_type()!=type_width_point
+	 && value_desc.get_value_node()->get_type()!=type_bline_point)
 	{
-		ValueBase components[6];
+		ValueNode_Composite::Handle composite_value_node = ValueNode_Composite::Handle::cast_dynamic(value_desc.get_value_node());
+		int index = value_desc.parent_is_value_desc()
+				  ? composite_value_node->get_link_index_from_name(value_desc.get_sub_name())
+				  : -1;
+
+		ValueBase components[8];
 		int n_components(0);
-		switch(value.get_type())
+		Type &type(value.get_type());
+		if (type == type_vector)
 		{
-		case ValueBase::TYPE_VECTOR:
 			components[0]=value.get(Vector())[0];
 			components[1]=value.get(Vector())[1];
 			n_components=2;
-			break;
-		case ValueBase::TYPE_COLOR:
+		}
+		else
+		if (type == type_color)
+		{
 			components[0]=value.get(Color()).get_r();
 			components[1]=value.get(Color()).get_g();
 			components[2]=value.get(Color()).get_b();
 			components[3]=value.get(Color()).get_a();
 			n_components=4;
-			break;
-		case ValueBase::TYPE_SEGMENT:
+		}
+		else
+		if (type == type_segment)
+		{
 			components[0]=value.get(Segment()).p1;
 			components[1]=value.get(Segment()).t1;
 			components[2]=value.get(Segment()).p2;
 			components[3]=value.get(Segment()).t2;
 			n_components=4;
-			break;
-		case ValueBase::TYPE_BLINEPOINT:
-			{
-				BLinePoint bline_point(value);
-				components[0]=bline_point.get_vertex();
-				components[1]=bline_point.get_width();
-				components[2]=bline_point.get_origin();
-				components[3]=bline_point.get_split_tangent_flag();
-				components[4]=bline_point.get_tangent1();
-				components[5]=bline_point.get_tangent2();
-				n_components=6;
-				break;
-			}
-		default:
-			throw Error(_("Bad type for composite (%s)"),ValueBase::type_local_name(value.get_type()).c_str());
-			break;
 		}
+		else
+		if (type == type_transformation)
+		{
+			components[0]=value.get(Transformation()).offset;
+			components[1]=value.get(Transformation()).angle;
+			components[2]=value.get(Transformation()).skew_angle;
+			components[3]=value.get(Transformation()).scale;
+			n_components=4;
+		}
+		else
+		if (type == type_bline_point)
+		{
+			components[0]=value.get(BLinePoint()).get_vertex();
+			components[1]=value.get(BLinePoint()).get_width();
+			components[2]=value.get(BLinePoint()).get_origin();
+			components[3]=value.get(BLinePoint()).get_split_tangent_both();
+			components[4]=value.get(BLinePoint()).get_tangent1();
+			components[5]=value.get(BLinePoint()).get_tangent2();
+			components[6]=value.get(BLinePoint()).get_split_tangent_radius();
+			components[7]=value.get(BLinePoint()).get_split_tangent_angle();
+			n_components=8;
+		}
+		else
+		if (dynamic_cast<synfig::types_namespace::TypeWeightedValueBase*>(&type) != NULL)
+		{
+			types_namespace::TypeWeightedValueBase *tp =
+				dynamic_cast<synfig::types_namespace::TypeWeightedValueBase*>(&type);
+			components[0]=tp->extract_weight(value);
+			components[1]=tp->extract_value(value);
+			n_components=2;
+		}
+		else
+		if (dynamic_cast<synfig::types_namespace::TypePairBase*>(&type) != NULL)
+		{
+			types_namespace::TypePairBase *tp =
+				dynamic_cast<synfig::types_namespace::TypePairBase*>(&type);
+			components[0]=tp->extract_first(value);
+			components[1]=tp->extract_second(value);
+			n_components=2;
+		}
+		else
+			throw Error(_("Bad type for composite (%s)"), type.description.local_name.c_str());
+
 		for(int i=0;i<n_components;i++)
 		{
-			ValueDesc component_value_desc(ValueNode_Composite::Handle::cast_dynamic(value_desc.get_value_node()),i);
+			if (index < 0 || index == i)
+			{
+				ValueDesc component_value_desc(ValueNode_Composite::Handle::cast_dynamic(value_desc.get_value_node()),i);
+				Action::Handle action(Action::create("ValueDescSet"));
+				if(!action)
+					throw Error(_("Unable to find action ValueDescSet (bug)"));
+				action->set_param("canvas",get_canvas());
+				action->set_param("canvas_interface",get_canvas_interface());
+				action->set_param("time",time);
+				action->set_param("new_value",components[i]);
+				action->set_param("value_desc",component_value_desc);
+				if(!action->is_ready())
+					throw Error(Error::TYPE_NOTREADY);
+				add_action(action);
+			}
+		}
+		return;
+	}
+
+	// If we are a composite value node type BLINEPOINT, then
+	// we need to distribute the changes to the
+	// individual parts in a proper way
+	if(value_desc.is_value_node() && ValueNode_Composite::Handle::cast_dynamic(value_desc.get_value_node())
+	 && value_desc.get_value_node()->get_type()==type_bline_point)
+	{
+		ValueNode_Composite::Handle composite_value_node = ValueNode_Composite::Handle::cast_dynamic(value_desc.get_value_node());
+		int index1 = value_desc.parent_is_value_desc()
+				  ? composite_value_node->get_link_index_from_name(value_desc.get_sub_name())
+				  : -1;
+		int index2 = value_desc.parent_is_value_desc()
+				  && value_desc.get_sub_name() == "t2"
+				  && (!value.get(BLinePoint()).get_split_tangent_radius() || !value.get(BLinePoint()).get_split_tangent_angle())
+				  ? composite_value_node->get_link_index_from_name("t1")
+				  : -1;
+
+		ValueBase components[8];
+		int n_components(0);
+		int order[8] = { 0,1,2,3,6,7,4,5 };
+		components[0]=value.get(BLinePoint()).get_vertex();
+		components[1]=value.get(BLinePoint()).get_width();
+		components[2]=value.get(BLinePoint()).get_origin();
+		components[3]=value.get(BLinePoint()).get_split_tangent_both();
+		components[4]=value.get(BLinePoint()).get_tangent1();
+		components[5]=value.get(BLinePoint()).get_tangent2();
+		components[6]=value.get(BLinePoint()).get_split_tangent_radius();
+		components[7]=value.get(BLinePoint()).get_split_tangent_angle();
+		n_components=8;
+		for(int i=0;i<n_components;i++)
+		{
+			if (index1 < 0 || index1 == order[i] || index2 == order[i])
+			{
+				ValueDesc component_value_desc(ValueNode_Composite::Handle::cast_dynamic(value_desc.get_value_node()),order[i]);
+				Action::Handle action(Action::create("ValueDescSet"));
+				if(!action)
+					throw Error(_("Unable to find action ValueDescSet (bug)"));
+				action->set_param("canvas",get_canvas());
+				action->set_param("canvas_interface",get_canvas_interface());
+				action->set_param("time",time);
+				action->set_param("new_value",components[order[i]]);
+				action->set_param("value_desc",component_value_desc);
+				action->set_param("recursive", false);
+				if(!action->is_ready())
+					throw Error(Error::TYPE_NOTREADY);
+				add_action(action);
+			}
+		}
+		return;
+	}
+
+	// If we are a bone value node, then
+	// we need to distribute the changes to the
+	// individual parts except 'parent' and 'name' fields
+	if (value.get_type() == type_bone_object
+	 && ValueNode_Bone::Handle::cast_dynamic(value_desc.get_value_node()))
+	{
+		ValueNode_Bone::Handle bone_value_node =
+			ValueNode_Bone::Handle::cast_dynamic(
+				value_desc.get_value_node() );
+		const Bone &bone = value.get(Bone());
+
+		typedef std::pair<String, ValueBase> KeyValue;
+		std::pair<String, ValueBase> values[] = {
+			KeyValue("origin",   bone.get_origin()),
+			KeyValue("angle",    bone.get_angle()),
+			KeyValue("scalelx",  bone.get_scalelx()),
+			KeyValue("width",    bone.get_width()),
+			KeyValue("scalex",   bone.get_scalex()),
+			KeyValue("tipwidth", bone.get_tipwidth()),
+			KeyValue("bone_depth",    bone.get_depth()),
+			KeyValue("length",   bone.get_length())
+		};
+
+		for(int i = 0; i < (int)(sizeof(values)/sizeof(values[0])); ++i)
+		{
+			ValueDesc component_value_desc(
+				bone_value_node,
+				bone_value_node->get_link_index_from_name(values[i].first));
 			Action::Handle action(Action::create("ValueDescSet"));
 			if(!action)
 				throw Error(_("Unable to find action ValueDescSet (bug)"));
 			action->set_param("canvas",get_canvas());
 			action->set_param("canvas_interface",get_canvas_interface());
 			action->set_param("time",time);
-			action->set_param("new_value",components[i]);
+			action->set_param("new_value",values[i].second);
 			action->set_param("value_desc",component_value_desc);
 			if(!action->is_ready())
 				throw Error(Error::TYPE_NOTREADY);
@@ -305,6 +548,7 @@ Action::ValueDescSet::prepare()
 		}
 		return;
 	}
+
 	// If we are a RADIAL composite value node, then
 	// we need to distribute the changes to the
 	// individual parts
@@ -312,9 +556,8 @@ Action::ValueDescSet::prepare()
 	{
 		ValueBase components[6];
 		int n_components(0);
-		switch(value.get_type())
-		{
-		case ValueBase::TYPE_VECTOR:
+		Type &type(value.get_type());
+		if (type == type_vector)
 		{
 			Angle old_angle = (*(ValueNode_RadialComposite::Handle::cast_dynamic(
 									 value_desc.get_value_node())->get_link("theta")))(time).get(Angle());
@@ -326,18 +569,18 @@ Action::ValueDescSet::prepare()
 			components[1]=old_angle + change;
 			n_components=2;
 		}
-			break;
-		case ValueBase::TYPE_COLOR:
+		else
+		if (type == type_color)
+		{
 			components[0]=value.get(Color()).get_y();
 			components[1]=value.get(Color()).get_s();
 			components[2]=value.get(Color()).get_hue();
 			components[3]=value.get(Color()).get_a();
 			n_components=4;
-			break;
-		default:
-			throw Error(_("Bad type for radial composite (%s)"),ValueBase::type_local_name(value.get_type()).c_str());
-			break;
 		}
+		else
+			throw Error(_("Bad type for radial composite (%s)"), type.description.local_name.c_str());
+
 		for(int i=0;i<n_components;i++)
 		{
 			ValueDesc component_value_desc(ValueNode_RadialComposite::Handle::cast_dynamic(value_desc.get_value_node()),i);
@@ -368,11 +611,11 @@ Action::ValueDescSet::prepare()
 			return;
 		}
 		ValueBase new_value;
-		if (value.get_type() == ValueBase::TYPE_ANGLE)
+		if (value.get_type() == type_angle)
 			new_value = scale_value_node->get_inverse(time, value.get(Angle()));
-		else if(value.get_type() == ValueBase::TYPE_VECTOR)
+		else if(value.get_type() == type_vector)
 			new_value = scale_value_node->get_inverse(time, value.get(Vector()));
-		else if(value.get_type() == ValueBase::TYPE_REAL)
+		else if(value.get_type() == type_real)
 			new_value = scale_value_node->get_inverse(time, value.get(Real()));
 		else
 			throw Error(_("Inverse manipulation of %s scale values not implemented in core."), value.type_name().c_str());
@@ -393,7 +636,7 @@ Action::ValueDescSet::prepare()
 	if (ValueNode_Range::Handle range_value_node = ValueNode_Range::Handle::cast_dynamic(value_desc.get_value_node()))
 	{
 		ValueBase new_value;
-		if (value.get_type() == ValueBase::TYPE_ANGLE)
+		if (value.get_type() == type_angle)
 			new_value = range_value_node->get_inverse(time, value.get(Angle()));
 		else
 			throw Error(_("Inverse manipulation of %s range values not implemented in core."), value.type_name().c_str());
@@ -414,9 +657,9 @@ Action::ValueDescSet::prepare()
 	if (ValueNode_Integer::Handle integer_value_node = ValueNode_Integer::Handle::cast_dynamic(value_desc.get_value_node()))
 	{
 		ValueBase new_value;
-		if (value.get_type() == ValueBase::TYPE_ANGLE)
+		if (value.get_type() == type_angle)
 			new_value = integer_value_node->get_inverse(time, value.get(Angle()));
-		else if(value.get_type() == ValueBase::TYPE_REAL)
+		else if(value.get_type() == type_real)
 			new_value = integer_value_node->get_inverse(time, value.get(Real()));
 		else
 			throw Error(_("Inverse manipulation of %s scale values not implemented in core."), value.type_name().c_str());
@@ -433,11 +676,11 @@ Action::ValueDescSet::prepare()
 		add_action(action);
 		return;
 	}
-	// Real: Reverse manipulations for Real->Ange convert
+	// Real: Reverse manipulations for Real->Angle convert
 	if (ValueNode_Real::Handle real_value_node = ValueNode_Real::Handle::cast_dynamic(value_desc.get_value_node()))
 	{
 		ValueBase new_value;
-		if (value.get_type() == ValueBase::TYPE_ANGLE)
+		if (value.get_type() == type_angle)
 			new_value = real_value_node->get_inverse(time, value.get(Angle()));
 		else
 			throw Error(_("Inverse manipulation of %s scale values not implemented in core."), value.type_name().c_str());
@@ -480,7 +723,7 @@ Action::ValueDescSet::prepare()
 	{
 		ValueNode_BLine::Handle bline = ValueNode_BLine::Handle::cast_dynamic(bline_vertex->get_link("bline"));
 		Real radius = 0.0;
-		ValueBase new_amount;
+		Real new_amount;
 		if (((*(bline_vertex->get_link("loop")))(time).get(bool())))
 		{
 			// The bline is looped. Animation may require an amount parameter
@@ -488,7 +731,7 @@ Action::ValueDescSet::prepare()
 			// not change drastically.
 			Real amount_old((*(bline_vertex->get_link("amount")))(time).get(Real()));
 
-			Real amount_new = synfig::find_closest_point((*bline)(time), value, radius, bline->get_loop());
+			Real amount_new = synfig::find_closest_point((*bline)(time), value.get(Vector()), radius, bline->get_loop());
 			Real difference = fmod( fmod(amount_new - amount_old, 1.0) + 1.0 , 1.0);
 			//fmod is called twice to avoid negative values
 			if (difference > 0.5)
@@ -496,17 +739,17 @@ Action::ValueDescSet::prepare()
 			new_amount = amount_old+difference;
 		}
 		else
-			new_amount = synfig::find_closest_point((*bline)(time), value, radius, bline->get_loop());
+			new_amount = synfig::find_closest_point((*bline)(time), value.get(Vector()), radius, bline->get_loop());
 		bool homogeneous((*(bline_vertex->get_link("homogeneous")))(time).get(bool()));
 		if(homogeneous)
-			new_amount=std_to_hom((*bline)(time), new_amount, ((*(bline_vertex->get_link("loop")))(time).get(bool())), bline->get_loop() );
+			new_amount=std_to_hom((*bline)(time), new_amount, (*(bline_vertex->get_link("loop")))(time).get(bool()), bline->get_loop() );
 		Action::Handle action(Action::create("ValueDescSet"));
 		if(!action)
 			throw Error(_("Unable to find action ValueDescSet (bug)"));
 		action->set_param("canvas",get_canvas());
 		action->set_param("canvas_interface",get_canvas_interface());
 		action->set_param("time",time);
-		action->set_param("new_value",new_amount);
+		action->set_param("new_value",ValueBase(new_amount));
 		action->set_param("value_desc",ValueDesc(bline_vertex, bline_vertex->get_link_index_from_name("amount")));
 		if(!action->is_ready())
 			throw Error(Error::TYPE_NOTREADY);
@@ -517,12 +760,11 @@ Action::ValueDescSet::prepare()
 	// to achieve the desired tangent
 	if (ValueNode_BLineCalcTangent::Handle bline_tangent = ValueNode_BLineCalcTangent::Handle::cast_dynamic(value_desc.get_value_node()))
 	{
-		ValueBase new_scale;
+		ValueBase new_scale(synfig::Real(0));
 		ValueDesc scale_value_desc(bline_tangent,bline_tangent->get_link_index_from_name("scale"));
 		ValueDesc offset_value_desc(bline_tangent,bline_tangent->get_link_index_from_name("offset"));
-		switch(value_desc.get_value_type())
-		{
-		case ValueBase::TYPE_REAL:
+		Type &type(value_desc.get_value_type());
+		if (type == type_real)
 		{
 			Real old_length = (*bline_tangent)(time).get(Real());
 			Real new_length = value.get(Vector()).mag();
@@ -531,13 +773,15 @@ Action::ValueDescSet::prepare()
 			if (fixed_length)
 			{
 				new_scale = new_length;
-				break;
 			}
-			if (old_length == 0)
-				return;
-			new_scale = new_length * scale / old_length;
+			else
+			{
+				if (old_length == 0) return;
+				new_scale = new_length * scale / old_length;
+			}
 		}
-		case ValueBase::TYPE_VECTOR:
+		else
+		if (type == type_vector)
 		{
 			Vector old_tangent = (*bline_tangent)(time).get(Vector());
 			Angle old_angle = old_tangent.angle();
@@ -550,8 +794,8 @@ Action::ValueDescSet::prepare()
 			if (fixed_length)
 			{
 				new_scale = new_length;
-				break;
 			}
+			else
 			if (old_length != 0)
 			{
 				new_scale = new_length * scale / old_length;
@@ -568,8 +812,8 @@ Action::ValueDescSet::prepare()
 				add_action(action);
 			}
 		}
-		break;
-		case ValueBase::TYPE_ANGLE:
+		else
+		if (type == type_angle)
 		{
 			Angle old_angle = (*bline_tangent)(time).get(Angle());
 			Angle new_angle = value.get(Angle());
@@ -587,10 +831,8 @@ Action::ValueDescSet::prepare()
 			add_action(action);
 			return;
 		}
-		default:
-			break;
-		}
-		if (new_scale)
+
+		if (new_scale.get(synfig::Real()) != 0)
 		{
 			Action::Handle action(Action::create("ValueDescSet"));
 			if(!action)
@@ -612,9 +854,9 @@ Action::ValueDescSet::prepare()
 	// WidthPoint Composite: adjust the width point position
 	// to achieve the desired point on bline
 	// (Code copied from BLineCalcVertex above)
-	if (value_desc.parent_is_linkable_value_node() && value_desc.get_parent_value_node()->get_type() == ValueBase::TYPE_LIST)
+	if (value_desc.parent_is_linkable_value_node() && value_desc.get_parent_value_node()->get_type() == type_list)
 	{
-		if(ValueNode_DynamicList::Handle::cast_dynamic(value_desc.get_parent_value_node())->get_contained_type() == ValueBase::TYPE_WIDTHPOINT)
+		if(ValueNode_DynamicList::Handle::cast_dynamic(value_desc.get_parent_value_node())->get_contained_type() == type_width_point)
 		{
 			ValueNode_WPList::Handle wplist=ValueNode_WPList::Handle::cast_dynamic(value_desc.get_parent_value_node());
 			if(wplist)
@@ -640,8 +882,8 @@ Action::ValueDescSet::prepare()
 							}
 						}
 					Real radius = 0.0;
-					ValueBase new_amount;
-					WidthPoint wp((*wpoint_composite)(time));
+					Real new_amount;
+					WidthPoint wp((*wpoint_composite)(time).get(WidthPoint()));
 					if (wplistloop)
 					{
 						// The wplist is looped. Animation may require a position parameter
@@ -652,7 +894,7 @@ Action::ValueDescSet::prepare()
 						// If it is homogeneous then convert it to standard
 						amount_old=homogeneous?hom_to_std((*bline)(time), amount_old, wplistloop, blineloop):amount_old;
 						// grab a new position given by duck's position on the bline
-						Real amount_new = synfig::find_closest_point((*bline)(time), value, radius, blineloop);
+						Real amount_new = synfig::find_closest_point((*bline)(time), value.get(Vector()), radius, blineloop);
 						// calculate the difference between old and new amounts
 						Real difference = fmod( fmod(amount_new - amount_old, 1.0) + 1.0 , 1.0);
 						//fmod is called twice to avoid negative values
@@ -661,7 +903,7 @@ Action::ValueDescSet::prepare()
 						// calculate a new value for the position
 						new_amount=amount_old+difference;
 						// restore the homogeneous value if needed
-						new_amount = homogeneous?ValueBase(std_to_hom((*bline)(time), new_amount, wplistloop, blineloop)):new_amount;
+						new_amount = homogeneous ? std_to_hom((*bline)(time), new_amount, wplistloop, blineloop) : new_amount;
 						// this is the difference between the new amount and the old amount inside the boundaries
 						Real bound_diff((wp.get_lower_bound() + new_amount*(wp.get_upper_bound()-wp.get_lower_bound()))-amount_old_b);
 						// add the new diff to the current amount
@@ -670,9 +912,9 @@ Action::ValueDescSet::prepare()
 					else
 					{
 						// grab a new amount given by duck's position on the bline
-						new_amount = synfig::find_closest_point((*bline)(time), value , radius, blineloop);
+						new_amount = synfig::find_closest_point((*bline)(time), value.get(Vector()), radius, blineloop);
 						// if it is homogeneous then convert to it
-						new_amount=homogeneous?ValueBase(std_to_hom((*bline)(time), new_amount, wplistloop, blineloop)):new_amount;
+						new_amount = homogeneous ? std_to_hom((*bline)(time), new_amount, wplistloop, blineloop) : new_amount;
 						// convert the value inside the boundaries
 						new_amount = wp.get_lower_bound()+new_amount*(wp.get_upper_bound()-wp.get_lower_bound());
 					}
@@ -682,7 +924,7 @@ Action::ValueDescSet::prepare()
 					action->set_param("canvas",get_canvas());
 					action->set_param("canvas_interface",get_canvas_interface());
 					action->set_param("time",time);
-					action->set_param("new_value",new_amount);
+					action->set_param("new_value",ValueBase(new_amount));
 					action->set_param("value_desc",ValueDesc(wpoint_composite, wpoint_composite->get_link_index_from_name("position")));
 					if(!action->is_ready())
 						throw Error(Error::TYPE_NOTREADY);
@@ -695,68 +937,6 @@ Action::ValueDescSet::prepare()
 
 	// end reverse manipulations
 
-	// If we are merging the tangents of a BLinePoint,
-	// we must also set the second tangent for things
-	// to interpolate properly
-	if (value_desc.parent_is_value_node() &&
-	    value_desc.get_parent_value_node()->get_type()==ValueBase::TYPE_BLINEPOINT &&
-	    value_desc.get_index()==3)
-	{
-		ValueNode_Composite::Handle parent_value_node;
-		parent_value_node=parent_value_node.cast_dynamic(value_desc.get_parent_value_node());
-		assert(parent_value_node);
-		// are we splitting or merging the tangents?
-	    if (value.get(bool()))
-	    {
-			// we are splitting tangents
-			Action::Handle action(Action::create("ValueDescSet"));
-			if(!action)
-				throw Error(_("Unable to find action ValueDescSet (bug)"));
-			action->set_param("canvas",get_canvas());
-			action->set_param("canvas_interface",get_canvas_interface());
-			action->set_param("time",time);
-			action->set_param("new_value",(*parent_value_node->get_link("t1"))(time));
-			action->set_param("value_desc",ValueDesc(parent_value_node,parent_value_node->get_link_index_from_name("t2")));
-			if(!action->is_ready())
-				throw Error(Error::TYPE_NOTREADY);
-			add_action(action);
-	    }
-	    else
-	    {
-			// we are merging tangents
-			// the merged tangent should be the average of the 2 tangents we're merging
-			ValueBase average(((Vector)((*parent_value_node->get_link("t1"))(time)) +
-							   (Vector)((*parent_value_node->get_link("t2"))(time))) / 2);
-			{
-				Action::Handle action(Action::create("ValueDescSet"));
-				if(!action)
-					throw Error(_("Unable to find action ValueDescSet (bug)"));
-				action->set_param("canvas",get_canvas());
-				action->set_param("canvas_interface",get_canvas_interface());
-				action->set_param("time",time);
-				action->set_param("new_value",average);
-				action->set_param("value_desc",ValueDesc(parent_value_node,parent_value_node->get_link_index_from_name("t1")));
-				if(!action->is_ready())
-					throw Error(Error::TYPE_NOTREADY);
-				add_action(action);
-			}
-			{
-				Action::Handle action(Action::create("ValueDescSet"));
-				if(!action)
-					throw Error(_("Unable to find action ValueDescSet (bug)"));
-				action->set_param("canvas",get_canvas());
-				action->set_param("canvas_interface",get_canvas_interface());
-				action->set_param("time",time);
-				action->set_param("new_value",average);
-				action->set_param("value_desc",ValueDesc(parent_value_node,parent_value_node->get_link_index_from_name("t2")));
-				if(!action->is_ready())
-					throw Error(Error::TYPE_NOTREADY);
-				add_action(action);
-			}
-	    }
-
-	}
-
 	// if value desc has parent value node and parent is composite widthpoint type and index is 4 or 5
 	// then we are changing the value of a widthpoint boundary.
 	// It is needed to check that we aren't doing the boundary range zero
@@ -767,7 +947,7 @@ Action::ValueDescSet::prepare()
 		parent_value_node=parent_value_node.cast_dynamic(value_desc.get_parent_value_node());
 		assert(parent_value_node);
 		int i=value_desc.get_index();
-		if(parent_value_node->get_type() == ValueBase::TYPE_WIDTHPOINT && (i==4 || i==5))
+		if(parent_value_node->get_type() == type_width_point && (i==4 || i==5))
 		{
 			ValueNode::Handle low(parent_value_node->get_link("lower_bound"));
 			ValueNode::Handle upp(parent_value_node->get_link("upper_bound"));
@@ -782,9 +962,27 @@ Action::ValueDescSet::prepare()
 			}
 		}
 	}
+
+	// If we are changing the z_depth range parameters
+	// send a signal to the canvas interface to say that the layer has changed
+	if(value_desc.parent_is_layer()
+	   &&
+	   (value_desc.get_param_name() =="z_range"
+		||
+		value_desc.get_param_name() =="z_range_position"
+		||
+		value_desc.get_param_name() =="z_range_depth")
+	   )
+	{
+		if (get_canvas_interface() && recursive)
+		{
+			get_canvas_interface()->signal_layer_z_range_changed()(value_desc.get_layer(),true);
+		}
+	}
+
 	// If we are in animate editing mode
 	// TODO: Can we replace local_value to value after all parameters will be converted into ValueBase type?
-	if(get_edit_mode()&MODE_ANIMATE && !value_desc.get_static())
+	if((animate || (get_edit_mode()&MODE_ANIMATE)) && !value_desc.get_static())
 	{
 		ValueNode_Animated::Handle& value_node(value_node_animated);
 		// If this value isn't a ValueNode_Animated, but
@@ -874,12 +1072,77 @@ Action::ValueDescSet::prepare()
 			}
 			else
 			if(ValueNode_Animated::Handle::cast_dynamic(value_desc.get_value_node()))
-				throw Error(_("You must be in Animate-Editing-Mode to directly manipulate this value"));
+			{
+				// If we are in not animate mode let's assume that the user wants to offset the
+				// animated value node by the difference.
+				// this is valid only for value types that allows it.
+				ValueNode_Animated::Handle animated=ValueNode_Animated::Handle::cast_dynamic(value_desc.get_value_node());
+				Waypoint waypoint;
+				Type &type=animated->get_type();
+				Type &value_type=value.get_type();
+				if(value_type==type &&
+					(  type == type_integer
+					|| type == type_angle
+					|| type == type_time
+					|| type == type_real
+					|| type == type_vector)
+					)
+				{
+					if (!lock_animation)
+					{
+						if ( !get_canvas_interface()
+						  || !get_canvas_interface()->get_ui_interface()
+						  || UIInterface::RESPONSE_OK != get_canvas_interface()->get_ui_interface()->confirmation(
+								 _("You are trying to edit animated parameter while Animation Mode is off.\n\nDo you want to apply offset to this animation?" ),
+								 _("Hint: You can hold Spacebar key while editing parameter to avoid this confirmation dialog."),
+								 _("No"),
+								 _("Yes"),
+								 synfigapp::UIInterface::RESPONSE_OK ))
+						{
+							throw Error(_("Cancelled by user"));
+						}
+					}
+
+					synfig::ValueNode_Animated::WaypointList::const_iterator iter;
+					for(iter=animated->waypoint_list().begin(); iter<animated->waypoint_list().end(); iter++)
+					{
+						waypoint=*iter;
+						ValueBase waypoint_value(waypoint.get_value());
+						if (type == type_integer)
+							waypoint_value=ValueBase(waypoint_value.get(int())+value.get(int())-(*animated)(time).get(int()));
+						else
+						if (type == type_angle)
+							waypoint_value=ValueBase(waypoint_value.get(Angle())+value.get(Angle())-(*animated)(time).get(Angle()));
+						else
+						if (type == type_time)
+							waypoint_value=ValueBase(waypoint_value.get(Time())+value.get(Time())-(*animated)(time).get(Time()));
+						else
+						if (type == type_real)
+							waypoint_value=ValueBase(waypoint_value.get(Real())+value.get(Real())-(*animated)(time).get(Real()));
+						else
+						if (type == type_vector)
+							waypoint_value=ValueBase(waypoint_value.get(Vector())+value.get(Vector())-(*animated)(time).get(Vector()));
+
+						waypoint.set_value(waypoint_value);
+						Action::Handle action(WaypointSet::create());
+						action->set_param("canvas",get_canvas());
+						action->set_param("canvas_interface",get_canvas_interface());
+						action->set_param("value_node",ValueNode::Handle(animated));
+						action->set_param("waypoint",waypoint);
+						if(!action->is_ready())
+							throw Error(Error::TYPE_NOTREADY);
+						add_action(action);
+					}
+					return;
+				}
+				else
+					throw Error(_("You must be in Animate-Editing-Mode to directly manipulate this value"));
+			}
 			else
 				throw Error(_("Direct manipulation of this ValueNode type is not yet supported"));
 		}
 		else
-		if(value_desc.parent_is_layer_param() && !value_desc.is_value_node())
+		if(value_desc.parent_is_layer() && !value_desc.is_value_node())
 		{
 			Action::Handle layer_param_set(LayerParamSet::create());
 			layer_param_set->set_param("canvas",get_canvas());
@@ -894,4 +1157,6 @@ Action::ValueDescSet::prepare()
 		}
 		throw Error(_("Unsupported ValueDesc type"));
 	}
+
+	
 }
