@@ -38,24 +38,24 @@
 #include <synfig/loadcanvas.h>
 #include <synfig/savecanvas.h>
 #include <synfig/filesystemnative.h>
-#include <synfig/valuenode_composite.h>
-#include <synfig/valuenode_radialcomposite.h>
-#include <synfig/valuenode_reference.h>
-#include <synfig/valuenode_boneinfluence.h>
-#include <synfig/valuenode_greyed.h>
-#include <synfig/valuenode_blinecalctangent.h>
-#include <synfig/valuenode_blinecalcvertex.h>
-#include <synfig/valuenode_blinecalcwidth.h>
-#include <synfig/valuenode_wplist.h>
-#include <synfig/valuenode_scale.h>
-#include <synfig/valuenode_range.h>
-#include <synfig/valuenode_integer.h>
-#include <synfig/valuenode_real.h>
-#include <synfig/valuenode_bonelink.h>
-#include <synfig/valuenode_average.h>
-#include <synfig/valuenode_weightedaverage.h>
-#include <synfig/layer_pastecanvas.h>
-#include <synfig/layer_bitmap.h>
+#include <synfig/valuenodes/valuenode_composite.h>
+#include <synfig/valuenodes/valuenode_radialcomposite.h>
+#include <synfig/valuenodes/valuenode_reference.h>
+#include <synfig/valuenodes/valuenode_boneinfluence.h>
+#include <synfig/valuenodes/valuenode_greyed.h>
+#include <synfig/valuenodes/valuenode_blinecalctangent.h>
+#include <synfig/valuenodes/valuenode_blinecalcvertex.h>
+#include <synfig/valuenodes/valuenode_blinecalcwidth.h>
+#include <synfig/valuenodes/valuenode_wplist.h>
+#include <synfig/valuenodes/valuenode_scale.h>
+#include <synfig/valuenodes/valuenode_range.h>
+#include <synfig/valuenodes/valuenode_integer.h>
+#include <synfig/valuenodes/valuenode_real.h>
+#include <synfig/valuenodes/valuenode_bonelink.h>
+#include <synfig/valuenodes/valuenode_average.h>
+#include <synfig/valuenodes/valuenode_weightedaverage.h>
+#include <synfig/layers/layer_pastecanvas.h>
+#include <synfig/layers/layer_bitmap.h>
 #include <synfig/target_scanline.h>
 #include "actions/valuedescexport.h"
 #include "actions/layerparamset.h"
